/*
 * Programmable remote controller with touch screen and auto power off
 * 14March2022
 * Prepared for Instructable by BobP
 * Extensive comments provided to allow better understanding of how to customize for 
 *     different remote control units.
 * Sketch is written for the Arduino Pro-Mini (clone) and the 2.4" TFT LCD shield
 * In the comments the Arduino Pro-Mini will be refered to as the MCU
 * Just for refernce, the 2.4" TFT LCD shield is usually sold as a shileld for the Arduino Uno
 * There is a lot of detailed information about the IR protocols contained witin the ".txt" 
 *    files that are in the IRlib2 library folders. Excellent background material if you are interrested.
 * The code as presented here is fully operational and has been used with the authors equipment. 
 * Three different remote control units have had select functions cloned and group into this single remote control unit.
 * DO a text search for the phrase ***CHANGE*** This will lead you to the areas that will be edited when the code
 *     is customized for different remotes. There are 4 areas to edit.
 *     
 */
#include <Adafruit_GFX.h>             // Core graphics library
#include <Adafruit_TFTLCD.h>          // Hardware-specific library
#include <TouchScreen.h>              // Handles the touch screen functions of the display
//#include <IRLibAll.h>               // Not used in final coding, see text of Instructable
#include <IRLibSendBase.h>            // Required by IRlib2
                                      // The IR libraries are from the IRlib2 master
                                      // You will ***CHANGE*** the IRLib protocols to be what is needed
                                      // for YOUR remote control codes
#include <IRLib_P01_NEC.h>            // A protocol specific to the authors need
#include <IRLib_P02_Sony.h>           // A protocol specific to the authors need
#include <IRLib_P05_Panasonic_Old.h>  // A protocol specific to the authors need
// Comment out the IRLib protocols included by the author if not needed
// User will add their specific IRLib protocol "includes" here
// 
#include <IRLibCombo.h>               //Required by IRlib2, wraps everything together

// Defines used for LCD control, assign MCU pin to an LCD display function
// This is how the programmable remote has been wired
#define LCD_CS A3
#define LCD_CD A2
#define LCD_WR A1
#define LCD_RD A0
#define LCD_RESET A4

// The next four "defines" deal with connections to the touch screen.
// Defines listed below work with the Teyleten-Robot display listed in the BOM, bought on Amazon
// These defines assign a display touch screen function to a specific pin of the MCU
// This is how the programmable remote has been wired 
#define YP A1  // define the touch screen "Y+" connection pin
#define XM A2  // define the touch screen "X-" connection pin
#define YM 7   // define the touch screen "Y-" connection pin
#define XP 6   // define the touch screen "X+" connection pin

// I found that a displays purchased from a vendor (not listed in the bill of materials)
// used a different pin assignment for the touch screen.
// If you find your touch screen is not responding, the manufacturer 
// may have used a different pin assignemnt for YP, XM,YM & XP
// Comment out the four defines listed above and enable these 4 lines instead by removing the comment marks

//#define YP A3  // define the touch screen "Y+" connection pin
//#define XM A2  // define the touch screen "X-" connection pin
//#define YM 9   // define the touch screen "Y-" connection pin
//#define XP 8   // define the touch screen "X+" connection pin

// When writing to the display a specific color will be specified.
// The library uses a hex value (hexadecimal) but it is convenient to use "names"
// Below are the Hex values for the "named" graphic colors used in this sketch
#define BLUE    0x001F
#define RED     0xF800
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

// The raw data received from the touch screen is not a pixel value but a unique number
// Within this sketch, these unique numbers will be mapped over to a specific pixel location on the screen
// The defines listed below give the raw data values "names" that making the sketch convenient to modify if needed.
// Define the Touch Screen max and min "raw" values for X and Y.

#define TS_MINX 212       //raw data value provided when the minimum "X" position of the screen is touched
#define TS_MINY 207       //raw data value provided when the minimum "Y" position of the screen is touched
#define TS_MAXX 937       //raw data value provided when the maximum "X" position of the screen is touched
#define TS_MAXY 936       //raw data value provided when the maximum "Y" position of the screen is touched

// If you find that your touch position on the screen does not match a specific button location
// the min and max values may need slight adjustment up or down
// Further down in the sketch you will find code where the raw data values can be displayed in order to make 
// fine adjustments as needed

// Again, a list of defines to make the sketch more "readable"

#define IRenablePin A5      // Used to enable/disable the IR driver circuit
#define IRPulsePin 3        //Output pin used to control the IR LED driver circuit
#define PwrControlPin 11    //Output pin used to control the power On/Off circuit

// Create an LCD object "tft"
// The "tft" object handles the display graphics etc. 
// It is using the MCU pins assigned earlier

Adafruit_TFTLCD tft(LCD_CS, LCD_CD, LCD_WR, LCD_RD, LCD_RESET);

// Create a touch screen object "ts"
// The "ts" object handles the touch screen functions
// It is using the MCU pins assigned earlier

TouchScreen ts = TouchScreen(XP, YP, XM, YM, 303);

// Note the 303 value in the line above is the measured resistance of the touch screen being used
// 303 ohms is a typical value

// Create an object "IRsend" used to transmit the IR codes
// This is the object that gets the job done!

IRsend IRsender;

// Set up and initialize some variables and constants used within the sketch
// Note that the PWRDelay value listed below can be changed to suit your need
// The PwrDelay value is in milliseconds. The value listed provides a delay of 25 seconds

const unsigned long PwrDelay = 25000;   // Delay in milliseconds until the MCU shuts down the input power
unsigned long StartTime = 0;          // used by auto shutdown
unsigned long CurrentTime = 0;        //  used by auto shutdown
unsigned long ElapsedTime = 0;        //  used by auto shutdown
unsigned long IdleTime = 0;
int row  = 0;      // row Index to use when drawing buttons
int col  = 0;      // column Index to use when drawing buttons
int posX = 0;      // "X" value used to set starting corner when drawing a button rectangle
int posY = 0;      // "Y" value used to set starting corner when drawing a button rectangle
int ColNum = 0;    // A variable to hold column number of the button that was pressed
int RowNum = 0;    // A variable to hold row number of the button that was pressed
int BP = 9;        // BP will hold the value (button number 0-20) of button that was pressed
int PX = 0;        // Variable to hold the mapped "X" touch position data
int PY = 0;        // Variable to hold the mapped "Y" touch position data
//Note-- p.x & p.y are the raw data values of the touched position

// The following array will hold TEXT appearing within each button
// You will ***CHANGE*** the text to be what YOU want to have printed in the 
// center of each button on the screen. It is easy to change here but you are limited to 
// a miaximum of 10 characers (including spaces. Note that there are exactly 10 characters between the " " quote
// marks. There must always be 10 characters betwen the " ". This will keep the text centered/aligned

String ButtonText[7][3] =
{
  {"  Cbl Pwr ",    "  TV Pwr  ",    " Rcvr Pwr "},
  {"  Vol +   ",    "  Vol -   ",    "  Mute    "},
  {"  Guide   ",    "  Page-Up ",    "  Page-Dn "},
  {"  DVR     ",    "  Up      ",    "  Down    "},
  {"  CH +    ",    "  CH -    ",    "  Last    "},
  {"  Jump >  ",    "  FF >>>  ",    "  FR <<<  "},
  {"  Play    ",    "  [Stop]  ",    " <SELECT> "},
};

// IR send codes for each button are listed below
// The values are in HEX so each will begin with 0x
// you will ***CHANGE*** these values to to be the values specific to the
// remotes you are cloning. The data you will use here was recorded when you built the IR receiver 
// as described in the Instructable.
// Note that for reference, in the comment of each line I have included the button number, 0-20,
// the button text, the protocol to be used, and the number of bits. This information is totally
// optional here but the button numbers, 0 thru 20 dose serve a a convenient reference

long RemoteCode[] =
{
  0x37C107,   // 0- Cable Pwr Panasonic Old(5)  (22)
  0x240C,     // 1- Vol Up    Sony(2) (15)
  0x36C127,   // 2- Guide     Panasonic Old(5)  (22)
  0x36C926,   // 3- DVR       Panasonic Old(5)  (22)
  0X377111,   // 4- CH+       Panasonic Old(5)  (22)
  0x37C906,   // 5- Jump      Panasonic Old(5)  (22)
  0x37990C,   // 6- Play      Panasonic Old(5)  (22)
  0x20DF10EF, // 7- TV Pwr    NEC(1)  (32)
  0x640C,     // 8- Vol Dn    Sony(2) (15)
  0x36D924,   // 9- Page Up   Panasonic Old(5)  (22)
  0x36812F,   // 10- Up       Panasonic Old(5)  (22)
  0X36F121,   // 11- CH-      Panasonic Old(5)  (22)
  0x36293A,   // 12- FF       Panasonic Old(5)  (22)
  0x365934,   // 13- STOP     Panasonic Old(5)  (22)
  0x540C,     // 14- Rcvr Pwr Sony(2) (15)
  0x140C,     // 15- Mute     Sony(2) (15)
  0x37D904,   // 16- Page dn  Panasonic Old(5)  (22)
  0x37A10B,   // 17- Dn       Panasonic Old(5)  (22)
  0x36E123,   // 18- Last     Panasonic Old(5)  (22)
  0x37291A,   // 19- Rew      Panasonic Old(5)  (22)
  0x366133,   // 20- Select   Panasonic Old(5)  (22)
};

// Button number to screen position mapping
// The 2-dimensional array below simply maps/assigns a button number
// to a specific position on the screen. Do not change

int Button[7][3] =
{
  {0, 7,  14},
  {1, 8,  15},
  {2, 9,  16},
  {3, 10, 17},
  {4, 11, 18},
  {5, 12, 19},
  {6, 13, 20},
};

//*********| SETUP BEGINS HERE |********
void setup()
{
// Uncomment (enable) line below if you want to view some certain values using the Serial Monitor
// Serial.begin(115200);               // This is only needed during setup or when trouble shooting
   pinMode(IRenablePin, OUTPUT);         // IR driver Enable/Disable control
  digitalWrite(IRenablePin, LOW);       // Preset to "LOW" Disables the IR driver ckt. at startup
  pinMode(IRPulsePin, OUTPUT);          // IR LED drive pin, also used by the LCD (this value is preset in IRLib2)
  digitalWrite(IRPulsePin, LOW);        // Initialize pin 3 level to turn off IR LED
  pinMode(PwrControlPin, OUTPUT);       // Used by DC power On/Off control
  digitalWrite(PwrControlPin, HIGH);    // Latch the power control circuit "ON" at startup

  //******|Initialize the LCD display|*******
  tft.reset();
 tft.begin(0x9341);         // Manually set the LCD identifier, the LCD specified use the ILI9341 driver chip
  tft.setRotation(2);        // Rotation can be a value from 0 to 3, changing this will rotate what is written to the display
  tft.setTextColor(YELLOW);  // Set the default text color to be yellow
  tft.setTextSize(1);        // Set default text size, "1" is smallest font
  tft.fillScreen(BLUE);                        // Paint the entire screen blue
  tft.drawRect(2, 2, 235, 316, WHITE);         // Draw a white border around the display
  tft.drawRect(3, 3, 233, 314, WHITE);         // Draw the Border again, offset by 1 pixel
  tft.fillRoundRect(30, 7, 180, 28, 8, RED);   // Draw a message area at top of screen, make it red
  tft.drawRoundRect(30, 7, 180, 28, 8, WHITE); // Draw white border arround message area

// Next draw an array of buttons on screen and print text within each button

  for (col = 0; col < 3; col++)                // Index for columns (X-direction)
  {
    for (row = 0; row < 7; row++)              // Index for the rows (Y-direction)
    {
      posX = (7 + (5 * col) + (72 * col));     // Calculate the "X" position of button
      posY = (38 + (5 * row) + (35 * row));    // Calculate the "Y" position of button
      tft.drawRect(posX, posY, 72, 35, WHITE); // Draw a botton 72 pixel wide by 35 high
      tft.setCursor(posX + 5, posY + 15);      // Set text position offset
      tft.println(ButtonText[row][col]);       // Get text from "ButtonText" array
    }
  }
   StartTime = millis();                       //set a time reference for auto power off timer
}
//***********| END of SETUP  |************************
//
// There is now a display screen with 21 buttons defined by a white border arround each button
// and text describing the bunton's function. The next section of code is where the touch screen is
// monitored for activity. The touched location is translated into a specific button number by using
// the Row and Column values. With the button number known, the desired remote control hex code value is obtained
// from the list (above) and, finally, this hex code is used to send pulses to the IR LED driver circuit.
// If there is touch screen activity the auto power off timer is constantly reset. If there is no touch screen activity
// for the specified amount of time, a command is sent to the power control circuit to turn the power OFF
//
//***********|  BEGIN LOOP   |************************
void loop()
{
 CurrentTime = millis();                 // Begin the power shutdown timing cycle
  RowNum = 9;                            // set the row number to an invalid value, used for data checking
  ColNum = 9;                            // set the column number to an invalid value, used for data checking
  digitalWrite (PwrControlPin, HIGH);    // Preset power shutdown control pin to a "Power ON" state
  Get_XY();                              // Get values X and Y of the touched position
  WhatColumn();                          // Use the touched position values to determine what Column
  WhatRow();                             // Use the touched position values to determine what Row
  WhatButton();                          // Use the row and column values to determine what button was pressed
  row = RowNum;
  col = ColNum;
  
// Check the if the power control timer should turn the power off
 if(CurrentTime - StartTime > PwrDelay and RowNum == 9) // Was the touch screen idle for more than the delay time?
     {
       digitalWrite(PwrControlPin, LOW); // If "yes", Set Power control pin low to to turn OFF the power
       delay(5000);                      // delay needed to allow power supply capacitor time to discharge
     }

// Setup to send an IR code
   if (RowNum != 9 && ColNum != 9)       // We have valid Row & Column values if they are not "9"
   {
    tft.setTextColor(YELLOW);            // Confirm by printing button text in the message area
    tft.setTextSize(2);                  // Using next larger size test here
    tft.setCursor(60, 14);               // Set text position for message, top center of screen
    tft.println(ButtonText[row][col]);   // Display name of button pressed in YELLOW
    SendCode(BP);                        // Use the "BP" (ButtonPressed) value to send the desired IR code, DO IT
    delay(550);                          // Delay attempts to reduce unwanted repeats
    digitalWrite(IRenablePin, LOW);      // DISABLE the IR circuit
    digitalWrite(IRPulsePin, LOW);       // Just to be sure, turn off the IR LED pulser
    tft.setCursor(60, 14);               // Setup to clear text from message area
    tft.setTextColor(RED);               // Temporarily change text color to be RED
    tft.println(ButtonText[row][col]);   // Clear message by overwriting the text
   }                                     // END routine to send the IR code

  // For trouble shooting, enable any or all 11 Serial.print lines as need 
 // The 11 lines below should be commented out before final compiling, for trouble shooting only
 //
 //    Serial.print("BP = ");               // 1
 //    Serial.println(BP);                  // 2
 //    Serial.print("RowNum = ");           // 3
 //    Serial.println(RowNum);              // 4
 //    Serial.print("ColNum = ");           // 5
 //    Serial.println(ColNum);              // 6
 //    Serial.print("Current Time is  ");   // 7
 //    Serial.println(CurrentTime);         // 8
 //     Serial.print("Start Time is  ");     // 9
 //     Serial.println(StartTime);           // 10
 //     delay(50);                           // 11
 //

  }  // END of LOOP
//***********| LOOP ENDS HERE |**********

//***********| Get Touch Screen values |*****
void Get_XY()
{
  PX = 0;                           // Reset "X" touch position value to zero (out of range)
  PY = 0;                           // Reset "Y" touch position value to zero (out of range
  delay(50);
  TSPoint p = ts.getPoint();        // Get the touch x/y/z data values
  pinMode(XM, OUTPUT);              // Since we are "sharing" pins with LCD & touch screen
  pinMode(YP, OUTPUT);              // must re-assert XM & YP as outputs

  if (p.z > 100 )                              // p.z is "pressure" value returned by the touch screen object "ts"
  {                                            // if it is greater than 100 we have good data to use
    PX = map(p.x, TS_MINX, TS_MAXX, 0, 240);   // Map the raw X "touch" value into a suitable "pixel" value, LCD width = 240 pixel
    PY = map(p.y, TS_MAXY, TS_MINY, 0, 320);   // Map the raw Y "touch" value into a suitable "pixel" value, LCD height = 320 pixel
  }

// You can uncomment (enable) all of the Serial.print lines below if needed
// These 15 lines below should be commented out before the final compiling, they are for trouble shooting only
  
  //  if (p.z > 100 && p.z < 700)   // 1 Only print values if the touch pressure, p.z, is greater than 100
  //  {                             // 2
  //    Serial.print("p.x = ");     // 3
  //    Serial.println(p.x);        // 4
  //    Serial.print("p.y = ");     // 5
  //    Serial.println(p.y);        // 6
  //    Serial.print("p.z = ");     // 7
  //    Serial.println(p.z);        // 8
  //    Serial.print("PX = ");      // 9
  //    Serial.println(PX);         // 10
  //    Serial.print("PY = ");      // 11
  //    Serial.println(PY);         // 12
  //    Serial.print("BP = ");      // 13
  //    Serial.println(BP);         // 14
  //   }                            // 15

}                                   //All done, return with the needed values

// **********| What column? | ***********
void WhatColumn()               // Use the "PX" touch value data to determine what "column" is being selected
{
  if (PX > 8 && PX < 75)        // Is the button in 1st col.?
  {
    ColNum = 0;                 // Set the ColNum var to "0"
  }
  else if (PX > 90 && PX < 152) // Is the button in 2nd col. ?
  {
    ColNum = 1;                  // Set the ColNum var to "1"
  }
  else if (PX > 173 && PX < 236) // was 236 Is the button in 3rd col. ?
  {
    ColNum = 2;                  // Set the ColNum var to "2"
  }
  else ColNum = 9;             // Set ColNum=9 to indicate invalid or no touch
  return ColNum;
}
// Done with Get_WhatColumn, return with the appropriate column number 0,1,2 or 9

//***********| What Row? |****************
void WhatRow()                   // Use the "PY" touch data values to determine what "row" is being selected
{
//  RowNum = 9;                    
  if (PY > 37 && PY < 67)        // Is the button in 1st row?
  {
    RowNum = 0;                  // set the RowNum var to "0"
  }
  else if (PY > 78 && PY < 108)  // Is the button in 2nd row?
  {
    RowNum = 1;                  // Set the RowNum var to "1"
  }
  else if (PY > 121 && PY < 148) // Is the button in 3rd row?
  {
    RowNum = 2;                  // Set the RowNum var to "2"
  }
  else if (PY > 161 && PY < 193) // Is the button in 4th row?
  {
    RowNum = 3;                  // Set the RowNum var to "3"
  }
  else if (PY > 206 && PY < 235) //  Is the button in 5th row?
  {
    RowNum = 4;                  // Set the RowNum var to "4"
  }
  else if (PY > 247 && PY < 277) // Is the button in 6th row?
  {
    RowNum = 5;                  // Set the RowNum var to "5"
  }
  else if (PY > 289 && PY < 318) // Is the button in 7th row?
  {
    RowNum = 6;                  // Set the RowNum var to "6"
  }
else RowNum = 9;                 // Set RowNum=9 to indicate not-valid or no touch
  return RowNum;              
}                               
// Done with WhatRow, return appropriate row number of 0-6 or 9

//**********|Button number|*********
void WhatButton()
{
 if (RowNum != 9 && ColNum != 9)        // We have valid Row & Column values if they are not "9"
  {
      BP = Button[RowNum][ColNum];      //  Use Row & Col. values to get "BP" (button Position)value from Button array
  }                                     // The button array was set up earlier
    return BP;                          // Return with the proper button number
 }
// Done with WhatButton, return appropriate button number


// **********| Send proper IR remote code |*******
// You must ***CHANGE*** the protocol name and the value for the number of bits to match YOUR remote codes
// As an example-
// The provided sketch has a line that will send a power on/off command to my Sony receiver
// It appears as---->  IRsender.send(SONY, RemoteCode[BP], 15);
// You must edit the PROTOCOL and number of bits as shown here-
//   IRsender.send(PROTOCOL, RemoteCode[BP], bits);
// you will replace the text "PROTOCOL" with the name of the protocol that is proper for the code you want to send
// You will replace the text "bits" with the value that is proper for the code you want to send
// The PROTOCOL and bits information is what you recorded using the IR receiver (see Instructable text)
// The other text in the line will remain unchanged.
// The revised line of code will begin with IRsender.send and end with a ; (semicolon)
// When changing the text for the protocol be careful to not remove the comma ( , ) that appears immediately after the protocol name
//
// In this section the the proper IR code pulses are sent to the IR LED driver on the Power Control breadboard/PCB
// Essentially the "input" here is the number that represents the button pressed
// This button number, "BP", is used to find the line of code needed to drive the IR transmitter

void SendCode(int BP)                            // Use the Button number to send the proper IR code
{
    StartTime = CurrentTime;                     // If we end up here the power timeout reference time is reset
        digitalWrite(IRenablePin, HIGH);         // ENABLE the IR LED pulser circuit

// Now send the IR code associated with the button number
switch (BP)
  {
  case 0:                                        // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for CABLE Pwr
  break;
  case   1:                                   // Send Remote Code
  IRsender.send(SONY, RemoteCode[BP], 15);          // for Vol Up
  break;
  case   2:                                   // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for Guide
  break;
  case   3:                                   // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for DVR
  break;
  case  4:                                   // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for CH +
  break;
  case  5:                                   // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for Jump
  break;
  case  6:                                   // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for Play
  break;
  case  7:                                   // Send Remote Code
  IRsender.send(NEC, RemoteCode[BP], 32);    // for TV Pwr
  break;
  case  8:                                   // Send Remote Code
  IRsender.send(SONY, RemoteCode[BP], 15);   // for Vol Dn
  break;
  case  9:                                   // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for Page Up
  break;
  case  10:                                  // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for Up
  break;
  case  11:                                  // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for CH-
  break;
  case  12:                                  // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for FF
  break;
  case  13:                                  // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for STOP
  break;
  case  14:                                  // Send Remote Code
  IRsender.send(SONY, RemoteCode[BP], 15);          // for Rcvr PWR
  break;
  case  15:                                  // Send Rmote Code
  IRsender.send(SONY, RemoteCode[BP], 15);          // for MUTE
  break;
  case  16:                                  // Send Rmote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for PAGE Dn
  break;
  case  17:                                  // Send Rmote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 15); // for Dn
  break;
  case  18:                                  // Send Rmote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 15); // for Last
  break;
  case  19:                                  // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 32); // for Rew
  break;  
  case  20:                                  // Send Remote Code
  IRsender.send(PANASONIC_OLD, RemoteCode[BP], 22); // for Select
  break; 
  }  
}
//End of SendCode
///////////////// END of SKETCH ////////////////////////////////
